# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .enriched_normalized_completion import EnrichedNormalizedCompletion
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GenerateResultData(UniversalBaseModel):
    completions: typing.List[EnrichedNormalizedCompletion] = pydantic.Field()
    """
    The generated completions. This will generally be a list of length one.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
