# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .search_result import SearchResult
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchNodeResultData(UniversalBaseModel):
    results_output_id: str
    results: typing.List[SearchResult] = pydantic.Field()
    """
    The results of the search. Each result represents a chunk that matches the search query.
    """

    text_output_id: str
    text: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
