# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .array_vellum_value import ArrayVellumValue
import typing
from .test_suite_run_execution_output import TestSuiteRunExecutionOutput
from .test_suite_run_execution_metric_result import TestSuiteRunExecutionMetricResult
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class TestSuiteRunExecution(UniversalBaseModel):
    id: str
    test_case_id: str
    outputs: typing.List[TestSuiteRunExecutionOutput]
    metric_results: typing.List[TestSuiteRunExecutionMetricResult]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
