# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .vembda_service_tier_enum import VembdaServiceTierEnum
from .quota import Quota
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class OrganizationLimitConfig(UniversalBaseModel):
    vembda_service_tier: typing.Optional[VembdaServiceTierEnum] = None
    prompt_executions_quota: typing.Optional[Quota] = None
    workflow_executions_quota: typing.Optional[Quota] = None
    workflow_runtime_seconds_quota: typing.Optional[Quota] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
