# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .folder_entity_folder_data import FolderEntityFolderData
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class FolderEntityFolder(UniversalBaseModel):
    """
    A slim representation of a Folder, as it exists within another Folder.
    """

    id: str
    type: typing.Literal["FOLDER"] = "FOLDER"
    data: FolderEntityFolderData

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
